Class: FxmlLoader
- Inherits:
-
Object
- Object
- FxmlLoader
- Defined in:
- lib/jrubyfx-fxmlloader.rb
Constant Summary collapse
- FX_NAMESPACE_VERSION =
"1"- @@fxml_jit_info =
{}
Instance Attribute Summary collapse
-
#builderFactory ⇒ Object
Returns the value of attribute builderFactory.
-
#charset ⇒ Object
Returns the value of attribute charset.
-
#controller ⇒ Object
Returns the value of attribute controller.
-
#controllerFactory ⇒ Object
Returns the value of attribute controllerFactory.
-
#current ⇒ Object
Returns the value of attribute current.
-
#location ⇒ Object
Returns the value of attribute location.
-
#namespace ⇒ Object
Returns the value of attribute namespace.
-
#root ⇒ Object
Returns the value of attribute root.
-
#scriptEngine ⇒ Object
Returns the value of attribute scriptEngine.
-
#staticLoad ⇒ Object
Returns the value of attribute staticLoad.
-
#template ⇒ Object
Returns the value of attribute template.
Instance Method Summary collapse
- #clearImports ⇒ Object
- #compareJFXVersions(rtVer, nsVer) ⇒ Object
- #createElement ⇒ Object
- #getScriptEngineManager ⇒ Object
- #getType(name) ⇒ Object
- #importClass(name) ⇒ Object
- #importPackage(name) ⇒ Object
-
#initialize(url = nil, ctrlr = nil, resourcs = nil, buildFactory = nil, charset = nil, loaders = nil) ⇒ FxmlLoader
constructor
A new instance of FxmlLoader.
-
#load(jruby_exts = {jruby_ext: {}}) ⇒ Object
either :no_jit, or a number above 0 representing the number of times before jitting a fxml file.
- #loadType(name, cache) ⇒ Object
- #loadTypeForPackage(packageName, className = nil) ⇒ Object
- #processCharacters ⇒ Object
- #processComment ⇒ Object
- #processEndElement ⇒ Object
- #processImport ⇒ Object
- #processLanguage ⇒ Object
- #processProcessingInstruction ⇒ Object
- #processStartElement ⇒ Object
Constructor Details
#initialize(url = nil, ctrlr = nil, resourcs = nil, buildFactory = nil, charset = nil, loaders = nil) ⇒ FxmlLoader
Returns a new instance of FxmlLoader.
270 271 272 273 274 275 276 277 278 279 280 |
# File 'lib/jrubyfx-fxmlloader.rb', line 270 def initialize(url=nil, ctrlr=nil, resourcs=nil, buildFactory=nil, charset=nil, loaders=nil) @location = url @builderFactory = buildFactory || JavaFXBuilderFactory.new @template = false @namespace = FXCollections.observableHashMap() self.controller = ctrlr @packages = [] @classes = {} @root = nil @charset = charset || Charset.forName(FXL::DEFAULT_CHARSET_NAME) end |
Instance Attribute Details
#builderFactory ⇒ Object
Returns the value of attribute builderFactory.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def builderFactory @builderFactory end |
#charset ⇒ Object
Returns the value of attribute charset.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def charset @charset end |
#controller ⇒ Object
Returns the value of attribute controller.
267 268 269 |
# File 'lib/jrubyfx-fxmlloader.rb', line 267 def controller @controller end |
#controllerFactory ⇒ Object
Returns the value of attribute controllerFactory.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def controllerFactory @controllerFactory end |
#current ⇒ Object
Returns the value of attribute current.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def current @current end |
#location ⇒ Object
Returns the value of attribute location.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def location @location end |
#namespace ⇒ Object
Returns the value of attribute namespace.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def namespace @namespace end |
#root ⇒ Object
Returns the value of attribute root.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def root @root end |
#scriptEngine ⇒ Object
Returns the value of attribute scriptEngine.
266 267 268 |
# File 'lib/jrubyfx-fxmlloader.rb', line 266 def scriptEngine @scriptEngine end |
#staticLoad ⇒ Object
Returns the value of attribute staticLoad.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def staticLoad @staticLoad end |
#template ⇒ Object
Returns the value of attribute template.
265 266 267 |
# File 'lib/jrubyfx-fxmlloader.rb', line 265 def template @template end |
Instance Method Details
#clearImports ⇒ Object
382 383 384 385 |
# File 'lib/jrubyfx-fxmlloader.rb', line 382 def clearImports @packages.clear @classes.clear end |
#compareJFXVersions(rtVer, nsVer) ⇒ Object
669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 |
# File 'lib/jrubyfx-fxmlloader.rb', line 669 def compareJFXVersions(rtVer, nsVer) retVal = 0; if (rtVer == nil || "" == (rtVer) || nsVer == nil || "" == (nsVer)) return retVal; end if (rtVer == (nsVer)) return retVal; end # version string can contain '-' dashIndex = rtVer.index("-"); dashIndex = -1 unless dashIndex if (dashIndex > 0) rtVer = rtVer[0, dashIndex] end # or "_" underIndex = rtVer.index("_"); underIndex = -1 unless underIndex if (underIndex > 0) rtVer = rtVer[0, underIndex] end # do not try to compare if the string is not valid version format if (!rtVer.match(/^(\d+)(\.\d+)*$/) || !nsVer.match(/^(\d+)(\.\d+)*$/)) return retVal; end nsVerTokenizer = StringTokenizer.new(nsVer, "."); rtVerTokenizer = StringTokenizer.new(rtVer, "."); nsDigit = 0 rtDigit = 0; rtVerEnd = false; while (nsVerTokenizer.hasMoreTokens() && retVal == 0) nsDigit = nsVerTokenizer.nextToken().to_i if (rtVerTokenizer.hasMoreTokens()) rtDigit = rtVerTokenizer.nextToken().to_i retVal = rtDigit - nsDigit; else rtVerEnd = true; break; end end if (rtVerTokenizer.hasMoreTokens() && retVal == 0) rtDigit = rtVerTokenizer.nextToken().to_i if (rtDigit > 0) retVal = 1; end end if (rtVerEnd) if (nsDigit > 0) retVal = -1; else while (nsVerTokenizer.hasMoreTokens()) nsDigit = nsVerTokenizer.nextToken().to_i if (nsDigit > 0) retVal = -1; break; end end end end return retVal; end |
#createElement ⇒ Object
448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 |
# File 'lib/jrubyfx-fxmlloader.rb', line 448 def createElement() prefix = @xmlStreamReader.getPrefix(); localName = @xmlStreamReader.getLocalName(); if !prefix i = localName.rindex('.') if localName[(i ? i : -1) + 1] == localName[(i ? i : -1) + 1].downcase name = localName[((i ? i : -1) + 1)..-1] if (i == nil) # This is an instance property if @loadListener @loadListener.beginPropertyElement(name, nil) end @current = PropertyElement.new(@current, @xmlStreamReader, @loadListener, self, name, nil) else # This is a static property sourceType = getType(localName[0, i]); if sourceType if @loadListener @loadListener.beginPropertyElement(name, sourceType); end @current = PropertyElement.new(@current, @xmlStreamReader, @loadListener, self,name, sourceType) elsif (@staticLoad) # The source type was not recognized if @loadListener @loadListener.beginUnknownStaticPropertyElement(localName); end @current = FXL::UnknownStaticPropertyElement.new else raise LoadException.new(localName + " is not a valid property."); end end else if (@current == nil && @root) raise LoadException.new("Root value already specified."); end type = getType(localName); prefixz = @xmlStreamReader.getLocation().getLineNumber().to_s + ": " numz = 1 pppn = @current while pppn numz+=1 pppn = pppn.parent end prefixz = (" " * numz) + prefixz if type if @loadListener @loadListener.beginInstanceDeclarationElement(type); end @current = InstanceDeclarationElement.new(@current, @xmlStreamReader, @loadListener, self, type) elsif (@staticLoad) # The type was not recognized if @loadListener @loadListener.beginUnknownTypeElement(localName); end @current = UnknownTypeElement.new(@current, @xmlStreamReader, @loadListener, self) else raise LoadException.new(localName + " is not a valid type."); end end elsif prefix == FXL::FX_NAMESPACE_PREFIX if localName == FXL::INCLUDE_TAG if @loadListener @loadListener.beginIncludeElement() end @current = IncludeElement.new(@current, @xmlStreamReader, @loadListener, self) elsif localName == FXL::REFERENCE_TAG if @loadListener @loadListener.beginReferenceElement end @current = ReferenceElement.new(@current, @xmlStreamReader, @loadListener, self) elsif localName == FXL::COPY_TAG if @loadListener @loadListener.beginCopyElement(); end @current = CopyElement.new(@current, @xmlStreamReader, @loadListener, self) elsif localName == FXL::ROOT_TAG if @loadListener @loadListener.beginRootElement(); end @current = RootElement.new(@current, @xmlStreamReader, @loadListener, self) elsif localName == FXL::SCRIPT_TAG if @loadListener @loadListener.beginScriptElement(); end @current = ScriptElement.new(@current, @xmlStreamReader, @loadListener, self) elsif localName == FXL::DEFINE_TAG if @loadListener @loadListener.beginDefineElement(); end @current = DefineElement.new(@current, @xmlStreamReader, @loadListener, self) else raise LoadException.new(prefix + ":" + localName + " is not a valid element."); end else raise LoadException.new("Unexpected namespace prefix: " + prefix + "."); end end |
#getScriptEngineManager ⇒ Object
655 656 657 658 659 660 661 662 |
# File 'lib/jrubyfx-fxmlloader.rb', line 655 def getScriptEngineManager() unless @scriptEngineManager @scriptEngineManager = Java.javax.script.ScriptEngineManager.new @scriptEngineManager.setBindings(SimpleBindings.new(@namespace)) end return @scriptEngineManager; end |
#getType(name) ⇒ Object
589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 |
# File 'lib/jrubyfx-fxmlloader.rb', line 589 def getType(name) type = nil if name[0] == name[0].downcase # This is a fully-qualified class name begin type = loadType(name, false); rescue ClassNotFoundException => exception # No-op end else # This is an unqualified class name type = @classes[name]; unless type # The class has not been loaded yet; look it up @packages.each do |packageName| begin type = loadTypeForPackage(packageName, name); rescue ClassNotFoundException => exception # No-op end break if type end unless type # check for ruby # TODO: this should require an import or something perhaps? need to think more about this? begin type = name.constantize_by(".") rescue # No-op end end @classes[name] = type if type end end return type; end |
#importClass(name) ⇒ Object
581 582 583 584 585 586 587 |
# File 'lib/jrubyfx-fxmlloader.rb', line 581 def importClass(name) begin loadType(name, true); rescue ClassNotFoundException => exception raise LoadException.new(exception); end end |
#importPackage(name) ⇒ Object
577 578 579 |
# File 'lib/jrubyfx-fxmlloader.rb', line 577 def importPackage(name) @packages << name end |
#load(jruby_exts = {jruby_ext: {}}) ⇒ Object
either :no_jit, or a number above 0 representing the number of times before jitting a fxml file
292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 |
# File 'lib/jrubyfx-fxmlloader.rb', line 292 def load(jruby_exts = {jruby_ext: {}}) @jruby_ext = {jit: 0}.merge(jruby_exts[:jruby_ext]) # TODO: actually open it properly unless jit_info = @@fxml_jit_info[file_path = @location.to_s] validate = true validate = @jruby_ext[:jit_validate] if @jruby_ext.has_key? :jit_validate jit_info = @@fxml_jit_info[file_path] = FxmlJitInfo.new(file_path, @jruby_ext[:jit], @jruby_ext[:jit_save_to], @jruby_ext[:jit_cache], validate, @jruby_ext[:jit_opts]) end inputStream = @location.open_stream if @template @root = nil else clearImports end @namespace[FXL::LOCATION_KEY] = @location @namespace[FXL::RESOURCES_KEY] = @resources @script_engine = nil $DEBUG_IT_FXML_RB = jit_info.should_jit? # if we have it cached, use the jitted method if jit_info.compiled? begin return :dont_load if @jruby_ext[:dont_load] return jit_info.__build_via_jit(@controller, @namespace, @jruby_ext) rescue Exception, java.lang.Throwable puts "JIT compiled method for #{@location.to_s} FAILED with error:" puts $! puts $!.backtrace puts "Reverting to normal parsing..." jit_info.jit_settings = :no_jit jit_info.decompile end end begin xmlInputFactory = XMLInputFactory.newFactory xmlInputFactory.setProperty("javax.xml.stream.isCoalescing", true) # Some stream readers incorrectly report an empty string as the prefix # for the default namespace; correct this as needed inputStreamReader = InputStreamReader.new(inputStream, @charset); @xmlStreamReader = SRDelegateClass.new(xmlInputFactory.createXMLStreamReader(inputStreamReader)) rescue XMLStreamException => e raise LoadException.new(e) end # Parse the XML stream begin while @xmlStreamReader.hasNext() event = @xmlStreamReader.next(); case event when XMLStreamConstants::PROCESSING_INSTRUCTION processProcessingInstruction when XMLStreamConstants::COMMENT processComment when XMLStreamConstants::START_ELEMENT processStartElement when XMLStreamConstants::END_ELEMENT processEndElement when XMLStreamConstants::CHARACTERS processCharacters end end rescue XMLStreamException => exception raise Exception.new(exception) end if @controller # TODO: initialize should be called here # Inject controller fields @controller.instance_variable_set("@" + FXL::LOCATION_KEY, @location) @controller.instance_variable_set("@" + FXL::RESOURCES_KEY, @resources) end if $DEBUG_IT_FXML_RB code = "#{rsem_out}\n#{rget @root}" jit_info.compile(code) end $RB_CMAPPER = {} $RB_IDMAPPER = {} $RB_MAPPER = {} $RB_PREFIX = "" $RB_NShowMAPPER = {} $last_elt = nil $RB_SCRIPT_ENGINE_MAPPINGS = {} @xmlStreamReader = nil return @root end |
#loadType(name, cache) ⇒ Object
629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 |
# File 'lib/jrubyfx-fxmlloader.rb', line 629 def loadType(name, cache) i = name.index('.'); n = name.length; while (i && i < n && name[i + 1] == name[i + 1].downcase) i = name.index('.', i + 1); end if (i == nil || i == n) raise ClassNotFoundException.new(); end packageName = name[0, i]; className = name[(i + 1)..-1]; type = loadTypeForPackage(packageName, className); if (cache) @classes[className] = type end return type; end |
#loadTypeForPackage(packageName, className = nil) ⇒ Object
664 665 666 667 668 |
# File 'lib/jrubyfx-fxmlloader.rb', line 664 def loadTypeForPackage(packageName, className=nil) packageName = (packageName + "." + className.gsub('.', '$')) if className #TODO: fix for ruby stuff return Java.java.lang.Class::forName(packageName, true, FXL::default_class_loader); end |
#processCharacters ⇒ Object
570 571 572 573 574 575 |
# File 'lib/jrubyfx-fxmlloader.rb', line 570 def processCharacters() # Process the characters if (!@xmlStreamReader.isWhiteSpace()) @current.processCharacters(); end end |
#processComment ⇒ Object
429 430 431 |
# File 'lib/jrubyfx-fxmlloader.rb', line 429 def processComment @loadListener.readComment(@xmlStreamReader.text) if @loadListener end |
#processEndElement ⇒ Object
560 561 562 563 564 565 566 567 568 |
# File 'lib/jrubyfx-fxmlloader.rb', line 560 def processEndElement() @current.processEndElement(); if @loadListener @loadListener.endElement(@current.value); end # Move up the stack @current = @current.parent; end |
#processImport ⇒ Object
415 416 417 418 419 420 421 422 423 424 425 426 427 |
# File 'lib/jrubyfx-fxmlloader.rb', line 415 def processImport target = @xmlStreamReader.getPIData().strip if @loadListener @loadListener.readImportProcessingInstruction(target) end if target.end_with?(".*") importPackage(target[0,target.length - 2]) else importClass(target) end end |
#processLanguage ⇒ Object
396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 |
# File 'lib/jrubyfx-fxmlloader.rb', line 396 def processLanguage if @scriptEngine raise LoadException.new("Page language already set.") end language = @xmlStreamReader.getPIData() if @loadListener @loadListener.readLanguageProcessingInstruction(language) end unless staticLoad scriptEngineManager = getScriptEngineManager() @scriptEngine = scriptEngineManager.getEngineByName(language) rputs_script @scriptEngine, language @scriptEngine.setBindings(scriptEngineManager.getBindings(), ScriptContext.ENGINE_SCOPE) end end |
#processProcessingInstruction ⇒ Object
387 388 389 390 391 392 393 394 |
# File 'lib/jrubyfx-fxmlloader.rb', line 387 def processProcessingInstruction piTarget = @xmlStreamReader.getPITarget().strip if piTarget == FXL::LANGUAGE_PROCESSING_INSTRUCTION processLanguage elsif piTarget == FXL::IMPORT_PROCESSING_INSTRUCTION processImport end end |
#processStartElement ⇒ Object
433 434 435 436 437 438 439 440 441 442 443 444 445 446 |
# File 'lib/jrubyfx-fxmlloader.rb', line 433 def processStartElement() # Create the element createElement(); # Process the start tag @current.processStartElement(); # Set the root value unless @root @root = @current.value; rputs @root, "__local_jruby_ext[:on_root_set].call(self) if __local_jruby_ext[:on_root_set]" @jruby_ext[:on_root_set].call(@root) if @jruby_ext[:on_root_set] end end |